@@ -12,7 +12,7 @@ from xpra.gtk_common.gobject_compat import import_gobject, import_gtk, import_gd
 from xpra.client.gtk_base.gtk_client_window_base import HAS_X11_BINDINGS, XSHAPE
 from xpra.gtk_common.quit import gtk_main_quit_really, gtk_main_quit_on_fatal_exceptions_enable
 from xpra.util import (
-    updict, pver, iround, flatten_dict, envbool, repr_ellipsized, csv, first_time,
+    updict, pver, iround, flatten_dict, envbool, repr_ellipsized, ellipsizer, csv, first_time,
     DEFAULT_METADATA_SUPPORTED, XPRA_OPENGL_NOTIFICATION_ID,
     )
 from xpra.os_util import (
@@ -214,6 +214,23 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         self.UI_watcher.add_fail_callback(UI_failed)
 
 
+    def get_vrefresh(self):
+        #try via GTK:
+        rates = {}
+        display = display_get_default()
+        if hasattr(display, "get_n_monitors"):
+            for m in range(display.get_n_monitors()):
+                monitor = display.get_monitor(m)
+                log("monitor %i (%s) refresh-rate=%i", m, monitor.get_model(), monitor.get_refresh_rate())
+                rates[m] = monitor.get_refresh_rate()
+        rate = -1
+        if rates:
+            rate = iround(min(rates.values())/1000)
+        if rate<30 or rate>250:
+            rate = UIXpraClient.get_vrefresh(self)
+        return rate
+
+
     def get_notifier_classes(self):
         #subclasses may add their toolkit specific variants
         #by overriding this method
@@ -303,6 +320,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         if color_str in ("auto", ""):
             from hashlib import md5
             try:
+                if envbool("XPRA_NOMD5", False):
+                    raise ValueError("md5 explicitly disabled")
                 m = md5()
             except ValueError:
                 from hashlib import sha1
@@ -744,6 +763,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
 
     def has_transparency(self):
+        if not envbool("XPRA_ALPHA", True):
+            return False
         screen = screen_get_default()
         if screen is None:
             return is_Wayland()
@@ -783,7 +804,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
     def make_cursor(self, cursor_data):
         #if present, try cursor ny name:
         display = display_get_default()
-        cursorlog("make_cursor: has-name=%s, has-cursor-types=%s, xscale=%s, yscale=%s, USE_LOCAL_CURSORS=%s",
+        cursorlog("make_cursor(%s) has-name=%s, has-cursor-types=%s, xscale=%s, yscale=%s, USE_LOCAL_CURSORS=%s",
+                  ellipsizer(cursor_data),
                   len(cursor_data)>=10, bool(cursor_types), self.xscale, self.yscale, USE_LOCAL_CURSORS)
         #named cursors cannot be scaled
         #(round to 10 to compare so 0.95 and 1.05 are considered the same as 1.0, no scaling):
@@ -834,8 +856,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 img = Image.frombytes("RGBA", (w, h), pixels, "raw", "BGRA", w*4, 1)
                 target = Image.new("RGBA", (fw, fh))
                 target.paste(img, (0, 0, w, h))
-                pixels = img.tobytes("raw", "BGRA")
-                cursor_pixbuf = get_pixbuf_from_data(pixels, True, w, h, w*4)
+                pixels = target.tobytes("raw", "BGRA")
+                cursor_pixbuf = get_pixbuf_from_data(pixels, True, fw, fh, fw*4)
             else:
                 cursorlog("scaling cursor from %ix%i to fixed OS size %ix%i", w, h, fw, fh)
                 cursor_pixbuf = pixbuf.scale_simple(fw, fh, INTERP_BILINEAR)
