@@ -1080,14 +1080,15 @@ class WindowClient(StubClientMixin):
 
     def _process_configure_override_redirect(self, packet):
         wid, x, y, w, h = packet[1:6]
-        window = self._id_to_window[wid]
+        window = self._id_to_window.get(wid)
         ax = self.sx(x)
         ay = self.sy(y)
         aw = max(1, self.sx(w))
         ah = max(1, self.sy(h))
         geomlog("_process_configure_override_redirect%s move resize window %s (id=%s) to %s",
                 packet[1:], window, wid, (ax,ay,aw,ah))
-        window.move_resize(ax, ay, aw, ah, -1)
+        if window:
+            window.move_resize(ax, ay, aw, ah, -1)
 
 
     def window_close_event(self, wid):
@@ -1114,7 +1115,7 @@ class WindowClient(StubClientMixin):
                 metadata = getattr(window, "_metadata", {})
                 log("window_close_event(%i) metadata=%s", wid, metadata)
                 class_instance = metadata.strlistget("class-instance", (None, None), 2, 2)
-                title = metadata.get("title", "")
+                title = metadata.strget("title", "")
                 log("window_close_event(%i) title=%s, class-instance=%s", wid, title, class_instance)
                 matching_title_close = [x for x in TITLE_CLOSEEXIT if x and title.startswith(x)]
                 close = None
@@ -1413,6 +1414,8 @@ class WindowClient(StubClientMixin):
                 window.eos()
             return
         x, y, width, height, coding, data, packet_sequence, rowstride = packet[2:10]
+        for v in (x, y, width, height):
+            assert isinstance(v, int), "coordinates must be ints, found %s (%s)" % (v, type(v))
         if not window:
             #window is gone
             def draw_cleanup():
