@@ -6,6 +6,7 @@
 import sys
 import time
 import os
+import shlex
 import socket
 from subprocess import PIPE, Popen
 
@@ -42,6 +43,7 @@ KEY_AUTH = envbool("XPRA_SSH_KEY_AUTH", True)
 PASSWORD_AUTH = envbool("XPRA_SSH_PASSWORD_AUTH", True)
 PASSWORD_RETRY = envint("XPRA_SSH_PASSWORD_RETRY", 2)
 assert PASSWORD_RETRY>=0
+MAGIC_QUOTES = envbool("XPRA_SSH_MAGIC_QUOTES", True)
 
 
 
@@ -225,6 +227,24 @@ class SSHProxyCommandConnection(SSHSocketConnection):
                 }
             }
 
+    def close(self):
+        try:
+            SSHSocketConnection.close(self)
+        except Exception:
+            #this can happen if the proxy command gets a SIGINT,
+            #it's closed already and we don't care
+            log("SSHProxyCommandConnection.close()", exc_info=True)
+
+def safe_lookup(config_obj, host):
+    try:
+        return config_obj.lookup(host)
+    except ImportError as e:
+        log("%s.lookup(%s)", config_obj, host, exc_info=True)
+        log.warn("Warning: unable to load SSH host config for %r:", host)
+        log.warn(" %s", e)
+        if isinstance(e, ModuleNotFoundError):
+            log.warn(" (looks like a 'paramiko' distribution packaging issue)")
+    return {}
 
 def ssh_paramiko_connect_to(display_desc):
     #plain socket attributes:
@@ -254,8 +274,13 @@ def ssh_paramiko_connect_to(display_desc):
         host_config = None
         if os.path.exists(user_config_file):
             with open(user_config_file) as f:
-                ssh_config.parse(f)
-            host_config = ssh_config.lookup(host)
+                try:
+                    ssh_config.parse(f)
+                except Exception as e:
+                    log("parse(%s)", user_config_file, exc_info=True)
+                    log.error("Error parsing '%s':", user_config_file)
+                    log.error(" %s", e)
+            host_config = safe_lookup(ssh_config, host)
             if host_config:
                 host = host_config.get("hostname", host)
                 username = host_config.get("user", username)
@@ -279,7 +304,7 @@ def ssh_paramiko_connect_to(display_desc):
                     transport = ssh_client.get_transport()
                     do_ssh_paramiko_connect_to(transport, host,
                                                username, password,
-                                               host_config or ssh_config.lookup("*"),
+                                               host_config or safe_lookup(ssh_config, "*"),
                                                keyfiles)
                     chan = paramiko_run_remote_xpra(transport, proxy_command, remote_xpra, socket_dir, display_as_args)
                     peername = (host, port)
@@ -301,6 +326,8 @@ def ssh_paramiko_connect_to(display_desc):
             proxy_username = display_desc.get("proxy_username", username)
             proxy_password = display_desc.get("proxy_password", password)
             sock = socket_connect(dtype, proxy_host, proxy_port)
+            if not sock:
+                fail("SSH proxy transport failed to connect to %s:%s" % (proxy_host, proxy_port))
             middle_transport = Transport(sock)
             middle_transport.use_compression(False)
             try:
@@ -308,10 +335,10 @@ def ssh_paramiko_connect_to(display_desc):
             except SSHException as e:
                 log("start_client()", exc_info=True)
                 raise InitException("SSH negotiation failed: %s" % e)
-            proxy_host_config = ssh_config.lookup(host)
+            proxy_host_config = safe_lookup(ssh_config, host)
             do_ssh_paramiko_connect_to(middle_transport, proxy_host,
                                        proxy_username, proxy_password,
-                                       proxy_host_config or ssh_config.lookup("*"),
+                                       proxy_host_config or safe_lookup(ssh_config, "*"),
                                        keyfiles)
             log("Opening proxy channel")
             chan_to_middle = middle_transport.open_channel("direct-tcpip", (host, port), ('localhost', 0))
@@ -325,7 +352,7 @@ def ssh_paramiko_connect_to(display_desc):
                 raise InitException("SSH negotiation failed: %s" % e)
             do_ssh_paramiko_connect_to(transport, host,
                                        username, password,
-                                       host_config or ssh_config.lookup("*"),
+                                       host_config or safe_lookup(ssh_config, "*"),
                                        keyfiles)
             chan = paramiko_run_remote_xpra(transport, proxy_command, remote_xpra, socket_dir, display_as_args)
 
@@ -342,6 +369,8 @@ def ssh_paramiko_connect_to(display_desc):
         #plain TCP connection to the server,
         #we open it then give the socket to paramiko:
         sock = socket_connect(dtype, host, port)
+        if not sock:
+            fail("SSH failed to connect to %s:%s" % (host, port))
         sockname = sock.getsockname()
         peername = sock.getpeername()
         log("paramiko socket_connect: sockname=%s, peername=%s", sockname, peername)
@@ -353,7 +382,7 @@ def ssh_paramiko_connect_to(display_desc):
             log("start_client()", exc_info=True)
             raise InitException("SSH negotiation failed: %s" % e)
         do_ssh_paramiko_connect_to(transport, host, username, password,
-                                   host_config or ssh_config.lookup("*"),
+                                   host_config or safe_lookup(ssh_config, "*"),
                                    keyfiles)
         chan = paramiko_run_remote_xpra(transport, proxy_command, remote_xpra, socket_dir, display_as_args)
         conn = SSHSocketConnection(chan, sock, sockname, peername, (host, port), socket_info)
@@ -399,11 +428,11 @@ def do_ssh_paramiko_connect_to(transport, host, username, password, host_config=
                 log("HostKeys.load(%s)", known_hosts, exc_info=True)
 
         log("host keys=%s", host_keys)
-        keys = host_keys.lookup(host)
+        keys = safe_lookup(host_keys, host)
         known_host_key = (keys or {}).get(host_key.get_name())
         def keyname():
             return host_key.get_name().replace("ssh-", "")
-        if host_key==known_host_key:
+        if known_host_key and host_key==known_host_key:
             assert host_key
             log("%s host key '%s' OK for host '%s'", keyname(), keymd5(host_key), host)
         else:
@@ -627,6 +656,9 @@ keymd5(host_key),
     if not transport.is_authenticated() and NONE_AUTH:
         auth_none()
 
+    if not transport.is_authenticated() and PASSWORD_AUTH and password:
+        auth_password()
+
     if not transport.is_authenticated() and AGENT_AUTH:
         auth_agent()
 
@@ -637,9 +669,6 @@ keymd5(host_key),
     if not transport.is_authenticated() and PASSWORD_AUTH:
         auth_interactive()
 
-    if not transport.is_authenticated() and PASSWORD_AUTH and password:
-        auth_password()
-
     if not transport.is_authenticated() and PASSWORD_AUTH and not password:
         for _ in range(1+PASSWORD_RETRY):
             password = input_pass("please enter the SSH password for %s@%s:" % (username, host))
@@ -718,7 +747,7 @@ def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_
         ssh_fail_cb = ssh_connect_failed
     sshpass_command = None
     try:
-        cmd = display_desc["full_ssh"]
+        cmd = list(display_desc["full_ssh"])
         kwargs = {}
         env = display_desc.get("env")
         kwargs["stderr"] = sys.stderr
@@ -756,8 +785,15 @@ def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_
         remote_cmd += "else echo \"no run-xpra command found\"; exit 1; fi"
         if INITENV_COMMAND:
             remote_cmd = INITENV_COMMAND + ";" + remote_cmd
-        #this gives us better compatibility with weird login shells:
-        cmd.append("sh -c '%s'" % remote_cmd)
+        #how many times we need to escape the remote command string
+        #depends on how many times the ssh command is parsed
+        nssh = sum(int(x=="ssh") for x in cmd)
+        if nssh>=2 and MAGIC_QUOTES:
+            for _ in range(nssh):
+                remote_cmd = shlex.quote(remote_cmd)
+        else:
+            remote_cmd = "'%s'" % remote_cmd
+        cmd.append("sh -c %s" % remote_cmd)
         if debug_cb:
             debug_cb("starting %s tunnel" % str(cmd[0]))
         #non-string arguments can make Popen choke,
