@@ -367,14 +367,14 @@ class ServerCore(object):
             if w and w.is_alive():
                 #wait up to 1 second for the worker thread to exit
                 try:
-                    w.wait(1)
+                    w.join(1)
                 except:
                     pass
                 if w.is_alive():
                     #still alive, force stop:
                     stop_worker(True)
                     try:
-                        w.wait(1)
+                        w.join(1)
                     except:
                         pass
             self.quit(upgrading)
@@ -582,11 +582,15 @@ class ServerCore(object):
         if www_dir:
             self._www_dir = www_dir
         else:
-            for ad,d in (
+            dirs = [
                 (get_resources_dir(), "html5"),
                 (get_resources_dir(), "www"),
                 (get_app_dir(), "www"),
-                ):
+                ]
+            if POSIX:
+                for d in ("/usr/share/xpra", "/usr/local/share/xpra"):
+                    dirs.append((d, "www"))
+            for ad,d in dirs:
                 self._www_dir = os.path.abspath(os.path.join(ad, d))
                 if os.path.exists(self._www_dir):
                     httplog("found html5 client in '%s'", self._www_dir)
@@ -1618,7 +1622,7 @@ class ServerCore(object):
             return
 
         log("process_hello: capabilities=%s", capabilities)
-        if c.boolget("version_request"):
+        if c.strget("request")=="version" or c.boolget("version_request"):
             self.send_version_info(proto, c.boolget("full-version-request"))
             return
         #verify version:
